<p class="logo">
  <a href="{{ pathto(master_doc) }}">
    <img class="logo" src="{{ pathto('_static/guessit-logo.png', 1) }}" alt="Logo"/>
  </a>
</p>
<p>
<iframe src="http://ghbtns.com/github-btn.html?user=wackou&repo=guessit&type=watch&count=true&size=large"
  allowtransparency="true" frameborder="0" scrolling="0" width="200px" height="35px"></iframe>
</p>

<p>
  GuessIt is a python library that tries to extract as much information
  as possible from a file.
</p>
