<p class="logo">
  <a href="{{ pathto(master_doc) }}">
    <img class="logo" src="{{ pathto('_static/guessit-logo.png', 1) }}" alt="Logo"/>
  </a>
</p>


<p>
  <iframe src="http://ghbtns.com/github-btn.html?user=wackou&repo=guessit&type=watch&count=true&size=large"
          allowtransparency="true" frameborder="0" scrolling="0" width="200px" height="35px"></iframe>
</p>

<p>
  <a href="https://pypi.python.org/pypi/guessit" target="_blank" class="nodots">
    <img src="http://img.shields.io/pypi/v/guessit.svg" alt="Build status" border="0" />
  </a>

  <a href="https://pypi.python.org/pypi/guessit" target="_blank" class="nodots">
    <img src="http://img.shields.io/badge/license-LGPLv3-blue.svg" alt="License" border="0" />
  </a>
</p>

<p>
  <a href="https://travis-ci.org/#!/wackou/guessit" target="_blank" class="nodots">
    <img src="http://img.shields.io/travis/wackou/guessit.svg" alt="Travis-CI" border="0" />
  </a>

  <a href="https://coveralls.io/r/wackou/guessit?branch=master" target="_blank" class="nodots">
    <img src="http://img.shields.io/coveralls/wackou/guessit.svg" alt="Code coverage" border="0" />
  </a>
</p>

<p>
  GuessIt is a python library that tries to extract and/or guess
  as much information as possible from a file.
</p>


<h3>Useful Links</h3>
<ul>
  <li><a class="dots" href="https://pypi.python.org/pypi/guessit">Guessit @ PyPI</a></li>
  <li><a class="dots" href="https://github.com/wackou/guessit">GuessIt @ GitHub</a></li>
  <li><a class="dots" href="https://github.com/wackou/guessit/issues">Issue Tracker</a></li>
</ul>

<h3>Donate</h3>
<p>
If you like GuessIt, please consider making a donation in
<a class="dots" href="http://bitcoin.org/">bitcoins</a> or in
<a class="dots" href="https://litecoin.org/">litecoins</a>.
</p>

<script src="_static/coinwidget/coin.js"></script>

<script>
CoinWidgetCom.go({
	wallet_address: "13fvWVf5oL28pJJ4uaruZc6f7fnQj4iCNa"
	, currency: "bitcoin"
	, counter: "hide"
	, alignment: "bl"
	, qrcode: false
	, auto_show: false
	, lbl_button: "Donate Bitcoins"
	, lbl_address: "GuessIt Bitcoin Donation Address:"
	, lbl_count: "donations"
	, lbl_amount: "BTC"
});
</script>

<script>
CoinWidgetCom.go({
	wallet_address: "LMkA2CvQSNLWiDT4yHvmbeFi5wbvDx5n9k"
	, currency: "litecoin"
	, counter: "hide"
	, alignment: "bl"
	, qrcode: false
	, auto_show: false
	, lbl_button: "Donate Litecoins"
	, lbl_address: "GuessIt Litecoin Donation Address:"
	, lbl_count: "donations"
	, lbl_amount: "LTC"
});
</script>

<br><br>
